# Generated by roxygen2: do not edit by hand

S3method(plot,performance)
export(performance)
export(prediction)
exportClasses(performance)
exportClasses(prediction)
exportMethods(plot)
import(graphics)
import(methods)
import(stats)
importFrom(grDevices,chull)
importFrom(grDevices,rainbow)
importFrom(grDevices,xy.coords)
importFrom(graphics,par)
importFrom(graphics,plot.default)
importFrom(graphics,plot.xy)
importFrom(stats,approxfun)
importFrom(stats,chisq.test)
importFrom(stats,median)
importFrom(stats,sd)
importFrom(stats,uniroot)
